﻿<#
    .SYNOPSIS
       Rename VM boot disk to match VM name and computer name.
    .DESCRIPTION
       Rename VM boot disk to match VM name and computer name.
       Works with Generation 1 and Generation 2 VMs.
       Custom Property 1 must be empty to invoke the VMs for renaming.
    .NOTES
        File Name : Invoke-SCVHDRename.ps1
        Author    : Daniel Neumann
        Requires  : PowerShell Version 3.0
                    Windows Server 2012 Hyper-V PowerShell Cmdlets
                    System Center 2012 Virtual Machine Manager PowerShell Cmdlets
        Version   : 1.0
    .LINK
        To provide feedback or for further assistance visit:
        http://www.danielstechblog.de  
    .EXAMPLE
       Invoke-SCVHDRename -VMMServer vmmserver.demo.local

       Call with the FQDN of the VMM Management Server.
    .EXAMPLE
       Invoke-SCVHDRename -VMMServer vmmserver

       Call with the name of the VMM Management Server.
#>
function Invoke-SCVHDRename
{
    param
    (
        [Parameter(Mandatory=$true, HelpMessage='VMM Server')]
        [String]
        $VMMServer
    )

    #Get-VMs
    $VM=Get-SCVirtualMachine -VMMServer $VMMServer|Where-Object {$_.CustomProperty.Custom1 -eq $null}
    foreach($VM in $VMs)
    {
        #Shutdown-VM
        if($VM.Status -eq "Running")
        {
            Stop-SCVirtualMachine -VM $VM -Shutdown
        }
        #Gen1
        if($VM.Generation -eq 1)
        {
            #Get Bootdisk
            $Disk=Get-SCVirtualDiskDrive -VM $VM|Where-Object {$_.BusType -eq "IDE" -and $_.Bus -eq 0 -and $_.Lun -eq 0}
            $VHD=Get-SCVirtualHardDisk -ID $Disk.VirtualHardDiskId
            #Check Replica
            $ReplicationEnabled=Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                param ($VM)
                if(Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name -ErrorAction SilentlyContinue){
                    $ReplicationEnabled="true"
                    $ReplicationEnabled
                }
            } -ArgumentList $VM
            #Remove Replica
            if($ReplicationEnabled -eq "true")
            {
                $ReplicaServer=Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                    param ($VM)
                    if(Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name)
                    {
                        $ReplicationSettings=Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name
                        $ReplicaServer=$ReplicationSettings.ReplicaServer
                        Remove-VMReplication -VMName $ReplicationSettings.Name
                        $ReplicaServer
                    }
                } -ArgumentList $VM
                Invoke-Command -ComputerName $ReplicaServer -ScriptBlock{
                    param ($VM)
                    Remove-VM -Name $VM.Name -Force
                } -ArgumentList $VM
                $VMHost=Get-SCVMHost -ComputerName $ReplicaServer
                Get-SCVirtualMachine -VMHost $VMHost -Name $VM.Name|Remove-SCVirtualMachine -Force
            }
            #VHD Rename
            Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                param($VM,$VHD) 
                    if((Test-Path -Path $VHD.Location) -and $VHD.Location -like "*.vhdx")
                    {
                        $NewName=$VM.Name+".vhdx"
                        $NewPath=$VHD.Directory+"\"+$NewName
                        Rename-Item -Path $VHD.Location -NewName $NewName
                        Set-VMHardDiskDrive -ComputerName $VM.HostName  -VMName $VM.Name -ControllerType IDE -Path $NewPath -ControllerNumber 0 -ControllerLocation 0 -AllowUnverifiedPaths -ErrorAction SilentlyContinue
                    }
                    if((Test-Path -Path $VHD.Location) -and $VHD.Location -like "*.vhd")
                    {
                        $NewName=$VM.Name+".vhd"
                        $NewPath=$VHD.Directory+"\"+$NewName
                        Rename-Item -Path $VHD.Location -NewName $NewName
                        Set-VMHardDiskDrive -ComputerName $VM.HostName  -VMName $VM.Name -ControllerType IDE -Path $NewPath -ControllerNumber 0 -ControllerLocation 0 -AllowUnverifiedPaths -ErrorAction SilentlyContinue       
                    }
            } -ArgumentList $VM,$VHD      
            #Enable Replica
            if($ReplicationEnabled -eq "true")
            {        
                Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                    param($VM,$ReplicaServer)
                    Enable-VMReplication -VMName $VM -ReplicaServerName $ReplicaServer -ReplicaServerPort 80 -AuthenticationType Kerberos -CompressionEnabled $true -RecoveryHistory 4 -VSSSnapshotFrequency 1 -AutoResynchronizeEnabled $true
                    Start-VMInitialReplication -VMName $VM
                } -ArgumentList $VM,$ReplicaServer
            }
            #VM Refresh
            Read-SCVirtualMachine -VM $VM
            #VM Update
            $CP1=Get-SCCustomProperty -Name Custom1
            Set-SCCustomPropertyValue -CustomProperty $CP1 -Value "Boot disk was renamed" -InputObject $VM
            Set-SCVirtualMachine -StopAction ShutdownGuestOS -VM $VM
            Read-SCVirtualMachine -VM $VM
            #Start VM
            if($VM.Status -eq "Running")
            {
                Start-SCVirtualMachine -VM $VM
            }
        }
        #Gen2
        if($VM.Generation -eq 2)
        {
            #Get Bootdisk
            $Disk=Get-SCVirtualDiskDrive -VM $VM|Where-Object {$_.BusType -eq "SCSI" -and $_.Bus -eq 0 -and $_.Lun -eq 0}
            $VHD=Get-SCVirtualHardDisk -ID $Disk.VirtualHardDiskId
            #Check Replica
            $ReplicationEnabled=Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                param ($VM)
                if(Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name -ErrorAction SilentlyContinue){
                    $ReplicationEnabled="true"
                    $ReplicationEnabled
                }
            } -ArgumentList $VM
            #Remove Replica
            if($ReplicationEnabled -eq "true")
            {
                $ReplicaServer=Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                    param ($VM)
                    if(Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name)
                    {
                        $ReplicationSettings=Get-VMReplication -ComputerName $VM.HostName -VMName $VM.Name
                        $ReplicaServer=$ReplicationSettings.ReplicaServer
                        Remove-VMReplication -VMName $ReplicationSettings.Name
                        $ReplicaServer
                    }
                } -ArgumentList $VM
                Invoke-Command -ComputerName $ReplicaServer -ScriptBlock{
                    param ($VM)
                    Remove-VM -Name $VM.Name -Force
                } -ArgumentList $VM
                $VMHost=Get-SCVMHost -ComputerName $ReplicaServer
                Get-SCVirtualMachine -VMHost $VMHost -Name $VM.Name|Remove-SCVirtualMachine -Force
            }
            #VHD Rename
            Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                    param($VM,$VHD) 
                    if((Test-Path -Path $VHD.Location) -and $VHD.Location -like "*.vhdx")
                    {
                        $NewName=$VM.Name+".vhdx"
                        $NewPath=$VHD.Directory+"\"+$NewName
                        Rename-Item -Path $VHD.Location -NewName $NewName
                        Set-VMHardDiskDrive -ComputerName $VM.HostName  -VMName $VM.Name -ControllerType SCSI -Path $NewPath -ControllerNumber 0 -ControllerLocation 0 -AllowUnverifiedPaths -ErrorAction SilentlyContinue  
                    }
            } -ArgumentList $VM,$VHD      
            #Enable Replica
            if($ReplicationEnabled -eq "true")
            {        
                Invoke-Command -ComputerName $VM.HostName -ScriptBlock{
                    param ($VM,$ReplicaServer)
                    Enable-VMReplication -VMName $VM -ReplicaServerName $ReplicaServer -ReplicaServerPort 80 -AuthenticationType Kerberos -CompressionEnabled $true -RecoveryHistory 4 -VSSSnapshotFrequency 1 -AutoResynchronizeEnabled $true
                    Start-VMInitialReplication -VMName $VM
                } -ArgumentList $VM,$ReplicaServer
            }
            #VM Refresh
            Read-SCVirtualMachine -VM $VM
            #VM Update
            $CP1=Get-SCCustomProperty -Name Custom1
            Set-SCCustomPropertyValue -CustomProperty $CP1 -Value "Boot disk was renamed" -InputObject $VM
            Set-SCVirtualMachine -StopAction ShutdownGuestOS -VM $VM
            Read-SCVirtualMachine -VM $VM
            #Start VM
            if($VM.Status -eq "Running")
            {
                Start-SCVirtualMachine -VM $VM
            }
        }
    }
}